package de.donmanfred;

import java.io.File;

import com.otaliastudios.cameraview.CameraListener;
import com.otaliastudios.cameraview.CameraOptions;
import com.otaliastudios.cameraview.CameraView;
import com.otaliastudios.cameraview.ExtraProperties;
import com.otaliastudios.cameraview.Facing;
import com.otaliastudios.cameraview.Flash;
import com.otaliastudios.cameraview.Grid;
import com.otaliastudios.cameraview.SessionType;
import com.otaliastudios.cameraview.VideoQuality;
import com.otaliastudios.cameraview.WhiteBalance;

import android.content.Context;
import android.graphics.PointF;
import android.hardware.Camera.Size;
import android.util.DisplayMetrics;
import android.view.ViewGroup;
import android.view.WindowManager.LayoutParams;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Pixel;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.CustomViewWrapper;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.ViewWrapper;
import anywheresoftware.b4a.objects.collections.Map;

@Version(1.01f)
@ShortName("CameraView")
//@Permissions(values={"android.permission.INTERNET", "android.permission.ACCESS_NETWORK_STATE"})
@Events(values={
		"onCameraOpened(options As Object)",
		"onCameraClosed()",
		"onPictureTaken(jpeg() As Byte)",
		"onVideoTaken(path As String, filename As String)",
		"onOrientationChanged(orientation As Int)",
		"onFocusStart(x As Int, y As Int)",
		"onFocusEnd(success As Boolean, x As Int, y As Int)",
		"onZoomChanged(newValue As Float, bounds() As Float, fingers() As Object)",
		"onExposureCorrectionChanged(newValue As Float, bounds() As Float, fingers() As Object)"
})
@DependsOn(values={"CameraView.aar"})
public class CameraViewwrapper extends ViewWrapper<CameraView>  implements DesignerCustomView {
	private BA ba;
	private String eventName;
	private CameraListener listener;
	 
	/*
	 * Initialize the HTML-TextView 
	 */   	
	public void Initialize(BA ba, String EventName) {
		_initialize(ba, null, EventName);
	}

	@Override
	public void DesignerCreateView(PanelWrapper base, LabelWrapper lw, Map props) {
		//getObject().setChecked((Boolean)props.Get("Checked"));
		CustomViewWrapper.replaceBaseWithView(base, getObject());
	}

	
	@SuppressWarnings("deprecation")
	@Hide
	@Override
	public void _initialize(final BA ba, Object activityClass, String EventName) {
		this.eventName = EventName.toLowerCase(BA.cul);
		this.ba = ba;
		this.setObject(new CameraView(ba.context));
		listener = new CameraListener(){
	    /**
	     * Notifies that the camera was opened.
	     * The {@link CameraOptions} object collects all supported options by the current camera.
	     *
	     * @param options camera supported options
	     */
	    @Override
	    public void onCameraOpened(CameraOptions options) {
				ba.raiseEventFromDifferentThread(this, null, 0, eventName + "_oncameraopened", true, new Object[] {options});	
	    }
	    /**
	     * Notifies that the camera session was closed.
	     */
	    @Override
	    public void onCameraClosed() {
				ba.raiseEventFromDifferentThread(this, null, 0, eventName + "_oncameraclosed", true, new Object[] {});	
	    }
	    /**
	     * Notifies that a picture previously captured with {@link CameraView#capturePicture()}
	     * or {@link CameraView#captureSnapshot()} is ready to be shown or saved.
	     *
	     * If planning to get a bitmap, you can use {@link CameraUtils#decodeBitmap(byte[], CameraUtils.BitmapCallback)}
	     * to decode the byte array taking care about orientation.
	     *
	     * @param jpeg captured picture
	     */
	    @Override
	    public void onPictureTaken(byte[] jpeg) {
				ba.raiseEventFromDifferentThread(this, null, 0, eventName + "_onpicturetaken", true, new Object[] {jpeg});	
	    }


	    /**
	     * Notifies that a video capture has just ended. The file parameter is the one that
	     * was passed to {@link CameraView#startCapturingVideo(File)}, if any.
	     * If not, the camera fallsback to:
	     * <code>
	     *     new File(getContext().getExternalFilesDir(null), "video.mp4");
	     * </code>
	     *
	     * @param video file hosting the mp4 video
	     */
	    @Override
	    public void onVideoTaken(File video) {
				ba.raiseEventFromDifferentThread(this, null, 0, eventName + "_onvideotaken", true, new Object[] {video.getPath(),video.getName()});	
	    }


	    /**
	     * Notifies that the device was tilted or the window offset changed.
	     * The orientation passed is exactly the counter-clockwise rotation that a View should have,
	     * in order to appear correctly oriented to the user, considering the way she is
	     * holding the device, and the native activity orientation.
	     *
	     * This is meant to be used for aligning views (e.g. buttons) to the current camera viewport.
	     *
	     * @param orientation either 0, 90, 180 or 270
	     */
	    @Override
	    public void onOrientationChanged(int orientation) {
				ba.raiseEventFromDifferentThread(this, null, 0, eventName + "_onorientationchanged", true, new Object[] {orientation});	

	    }


	    /**
	     * Notifies that user interacted with the screen and started focus with a gesture,
	     * and the autofocus is trying to focus around that area.
	     * This can be used to draw things on screen.
	     *
	     * @param point coordinates with respect to CameraView.getWidth() and CameraView.getHeight()
	     */
	    @Override
	    public void onFocusStart(PointF point) {
				ba.raiseEventFromDifferentThread(this, null, 0, eventName + "_onfocusstart", true, new Object[] {point.x,point.y});	
	    }


	    /**
	     * Notifies that a gesture focus event just ended, and the camera converged
	     * to a new focus (and possibly exposure and white balance).
	     * This might succeed or not.
	     *
	     * @param successful whether camera succeeded
	     * @param point coordinates with respect to CameraView.getWidth() and CameraView.getHeight()
	     */
	    @Override
	    public void onFocusEnd(boolean successful, PointF point) {
				ba.raiseEventFromDifferentThread(this, null, 0, eventName + "_onfocusend", true, new Object[] {successful,point.x,point.y});	
	    }


	    /**
	     * Noitifies that a finger gesture just caused the camera zoom
	     * to be changed. This can be used to draw, for example, a seek bar.
	     *
	     * @param newValue the new zoom value
	     * @param bounds min and max bounds for newValue (fixed to 0 ... 1)
	     * @param fingers finger positions that caused the event
	     */
	    @Override
	    public void onZoomChanged(float newValue, float[] bounds, PointF[] fingers) {
				ba.raiseEventFromDifferentThread(this, null, 0, eventName + "_onzoomchanged", true, new Object[] {newValue,bounds, fingers});	
	    }


	    /**
	     * Noitifies that a finger gesture just caused the camera exposure correction
	     * to be changed. This can be used to draw, for example, a seek bar.
	     *
	     * @param newValue the new correction value
	     * @param bounds min and max bounds for newValue, as returned by {@link CameraOptions}
	     * @param fingers finger positions that caused the event
	     */
	    @Override
	    public void onExposureCorrectionChanged(float newValue, float[] bounds, PointF[] fingers) {
				ba.raiseEventFromDifferentThread(this, null, 0, eventName + "_onexposurecorrectionchanged", true, new Object[] {newValue,bounds, fingers});	
	    }

		};
		getObject().setCameraListener(listener);
 	}

  /**
   * Returns whether the camera has started showing its preview.
   * @return whether the camera has started
   */
  public boolean getStarted() {
      return getObject().isStarted();
  }


  /**
   * Starts the camera preview, if not started already.
   * This should be called onResume(), or when you are ready with permissions.
   */
  public void start() {
  	getObject().start();
  }

  /**
   * Stops the current preview, if any was started.
   * This should be called onPause().
   */
  public void stop() {
  	getObject().stop();
  }

  public void destroy() {
  	getObject().destroy();
  }

  /**
   * Stops the current preview, if any was started.
   * This should be called onPause().
   */
  public void pause() {
  	getObject().stop();
  }

  // Public APIs for parameters and controls
  // ---------------------------------------


  /**
   * Returns a {@link CameraOptions} instance holding supported options for this camera
   * session. This might change over time. It's better to hold a reference from
   * {@link CameraListener#onCameraOpened(CameraOptions)}.
   *
   * @return an options map, or null if camera was not opened
   */
  public CameraOptions getCameraOptions() {
  	return getObject().getCameraOptions();
  }


  /**
   * If present, returns a collection of extra properties from the current camera
   * session.
   * @return an ExtraProperties object.
   */
  public ExtraProperties getExtraProperties() {
  	return getObject().getExtraProperties();
  }


  /**
   * Sets exposure adjustment, in EV stops. A positive value will mean brighter picture.
   *
   * If camera is not opened, this will have no effect.
   * If {@link CameraOptions#isExposureCorrectionSupported()} is false, this will have no effect.
   * The provided value should be between the bounds returned by {@link CameraOptions}, or it will
   * be capped.
   *
   * @see CameraOptions#getExposureCorrectionMinValue()
   * @see CameraOptions#getExposureCorrectionMaxValue()
   *
   * @param EVvalue exposure correction value.
   */
  public void setExposureCorrection(float EVvalue) {
  	getObject().setExposureCorrection(EVvalue);
  }


  /**
   * Returns the current exposure correction value, typically 0
   * at start-up.
   * @return the current exposure correction value
   */
  public float getExposureCorrection() {
  	return getObject().getExposureCorrection();
  }


  /**
   * Sets a zoom value. This is not guaranteed to be supported by the current device,
   * but you can take a look at {@link CameraOptions#isZoomSupported()}.
   * This will have no effect if called before the camera is opened.
   *
   * Zoom value should be between 0 and 1, where 1 will be the maximum available zoom.
   * If it's not, it will be capped.
   *
   * @param zoom value in [0,1]
   */
  public void setZoom(float zoom) {
  	getObject().setZoom(zoom);
  }


  /**
   * Returns the current zoom value, something between 0 and 1.
   * @return the current zoom value
   */
  public float getZoom() {
  	return getObject().getZoom();
  }


  /**
   * Controls the grids to be drawn over the current layout.
   *
   * @see Grid#OFF
   * @see Grid#DRAW_3X3
   * @see Grid#DRAW_4X4
   * @see Grid#DRAW_PHI
   *
   * @param gridMode desired grid mode
   */
  public void setGrid(String gridMode) {
  	getObject().setGrid(Grid.valueOf(gridMode));
  }


  /**
   * Gets the current grid mode.
   * @return the current grid mode
   */
  public Grid getGrid() {
  	return getObject().getGrid();
  }


  /**
   * Set location coordinates to be found later in the jpeg EXIF header
   *
   * @param latitude current latitude
   * @param longitude current longitude
   */
  public void setLocation(double latitude, double longitude) {
  	getObject().setLocation(latitude, longitude);
  }


  /**
   * Sets desired white balance to current camera session.
   *
   * @see WhiteBalance#AUTO
   * @see WhiteBalance#INCANDESCENT
   * @see WhiteBalance#FLUORESCENT
   * @see WhiteBalance#DAYLIGHT
   * @see WhiteBalance#CLOUDY
   *
   * @param whiteBalance desired white balance behavior.
   */
  public void setWhiteBalance(String whiteBalance) {
  	//WhiteBalance.AUTO
  	getObject().setWhiteBalance(WhiteBalance.valueOf(whiteBalance));
  }


  /**
   * Returns the current white balance behavior.
   * @return white balance value.
   */
  public WhiteBalance getWhiteBalance() {
  	return getObject().getWhiteBalance();
  }


  /**
   * Sets which camera sensor should be used.
   *
   * @see Facing#FRONT
   * @see Facing#BACK
   *
   * @param facing a facing value.
   */
  public void setFacing(final String facing) {
  	getObject().setFacing(Facing.valueOf(facing));
  }


  /**
   * Gets the facing camera currently being used.
   * @return a facing value.
   */
  public Facing getFacing() {
  	return getObject().getFacing();
  }


  /**
   * Toggles the facing value between {@link Facing#BACK}
   * and {@link Facing#FRONT}.
   *
   * @return the new facing value
   */
  public Facing toggleFacing() {
  	Facing facing = getObject().toggleFacing();
  	return facing;
  }


  /**
   * Sets the flash mode.
   *
   * @see Flash#OFF
   * @see Flash#ON
   * @see Flash#AUTO
   * @see Flash#TORCH

   * @param flash desired flash mode.
   */
  public void setFlash(String flash) {
  	getObject().setFlash(Flash.valueOf(flash));
  }


  /**
   * Gets the current flash mode.
   * @return a flash mode
   */
  public Flash getFlash() {
  	return getObject().getFlash();
  }


  /**
   * Toggles the flash mode between {@link Flash#OFF},
   * {@link Flash#ON} and {@link Flash#AUTO}, in this order.
   *
   * @return the new flash value
   */
  public Flash toggleFlash() {
  	Flash flash = getObject().toggleFlash();
  	return flash;
  }


  /**
   * Starts an autofocus process at the given coordinates, with respect
   * to the view width and height.
   *
   * @param x should be between 0 and getWidth()
   * @param y should be between 0 and getHeight()
   */
  public void startAutoFocus(float x, float y) {
  	getObject().startAutoFocus(x, y);
  }


  /**
   * Set the current session type to either picture or video.
   * When sessionType is video,
   * - {@link #startCapturingVideo(File)} will not throw any exception
   * - {@link #capturePicture()} might fallback to {@link #captureSnapshot()} or might not work
   *
   * @see SessionType#PICTURE
   * @see SessionType#VIDEO
   *
   * @param sessionType desired session type.
   */
  public void setSessionType(String sessionType) {
  	getObject().setSessionType(SessionType.valueOf(sessionType));
  }

  public void setSessionVideo() {
  	getObject().setSessionType(SessionType.VIDEO);
  }
  public void setSessionPicture() {
  	getObject().setSessionType(SessionType.PICTURE);
  }

  
  /**
   * Gets the current session type.
   * @return the current session type
   */
  public SessionType getSessionType() {
  	return getObject().getSessionType();
  }


  /**
   * Sets video recording quality. This is not guaranteed to be supported by current device.
   * If it's not, a lower quality will be chosen, until a supported one is found.
   * If sessionType is video, this might trigger a camera restart and a change in preview size.
   *
   * @see VideoQuality#LOWEST
   * @see VideoQuality#HIGHEST
   * @see VideoQuality#MAX_QVGA
   * @see VideoQuality#MAX_480P
   * @see VideoQuality#MAX_720P
   * @see VideoQuality#MAX_1080P
   * @see VideoQuality#MAX_2160P
   *
   * @param videoQuality requested video quality
   */
  public void setVideoQuality(String videoQuality) {
  	getObject().setVideoQuality(VideoQuality.valueOf(videoQuality));
  }


  /**
   * Gets the current video quality.
   * @return the current video quality
   */
  public String getVideoQuality() {
  	return getObject().getVideoQuality().name();
  }


  /**
   * Sets the JPEG compression quality for image outputs.
   * @param jpegQuality a 0-100 integer.
   */
  public void setJpegQuality(int jpegQuality) {
  	getObject().setJpegQuality(jpegQuality);
  }


  /**
   * Whether we should crop the picture output to match CameraView aspect ratio.
   * This is only relevant if CameraView dimensions were somehow constrained
   * (e.g. by fixed value or MATCH_PARENT) and do not match internal aspect ratio.
   *
   * Please note that this requires additional computations after the picture is taken.
   *
   * @param cropOutput whether to crop
   */
  public void setCropOutput(boolean cropOutput) {
  	getObject().setCropOutput(cropOutput);
  }



  /**
   * Adds a {@link CameraListener} instance to be notified of all
   * interesting events that happen during the camera lifecycle.
   *
   * @param cameraListener a listener for events.
   */
//  public void addCameraListener(CameraListener cameraListener) {
//      if (cameraListener != null) {
//          mCameraCallbacks.addListener(cameraListener);
//      }
//  }


  /**
   * Remove a {@link CameraListener} that was previously registered.
   *
   * @param cameraListener a listener for events.
   */
//  public void removeCameraListener(CameraListener cameraListener) {
//      if (cameraListener != null) {
//          mCameraCallbacks.removeListener(cameraListener);
//      }
//  }


  /**
   * Clears the list of {@link CameraListener} that are registered
   * to camera events.
   */
  public void clearCameraListeners() {
  	getObject().clearCameraListeners();
  }


  /**
   * Asks the camera to capture an image of the current scene.
   * This will trigger {@link CameraListener#onPictureTaken(byte[])} if a listener
   * was registered.
   *
   * Note that if sessionType is {@link SessionType#VIDEO}, this
   * might fall back to {@link #captureSnapshot()} (that is, we might capture a preview frame).
   *
   * @see #captureSnapshot()
   */
  public void capturePicture() {
  	getObject().capturePicture();
  }


  /**
   * Asks the camera to capture a snapshot of the current preview.
   * This eventually triggers {@link CameraListener#onPictureTaken(byte[])} if a listener
   * was registered.
   *
   * The difference with {@link #capturePicture()} is that this capture is faster, so it might be
   * better on slower cameras, though the result can be generally blurry or low quality.
   *
   * @see #capturePicture()
   */
  public void captureSnapshot() {
  	getObject().captureSnapshot();
  }


  /**
   * Starts recording a video with selected options, in a file called
   * "video.mp4" in the default folder.
   * This is discouraged, please use {@link #startCapturingVideo(File)} instead.
   *
   * @deprecated see {@link #startCapturingVideo(File)}
   */
//  @Deprecated
//  public void startCapturingVideo() {
//      startCapturingVideo(null);
//  }


  /**
   * Starts recording a video with selected options. Video will be written to the given file,
   * so callers should ensure they have appropriate permissions to write to the file.
   *
   * @param file a file where the video will be saved
   */
  public void startCapturingVideo(String path, String filename) {
  	File file = new File(path,filename);
  	getObject().startCapturingVideo(file);
  }


  /**
   * Starts recording a video with selected options. Video will be written to the given file,
   * so callers should ensure they have appropriate permissions to write to the file.
   * Recording will be automatically stopped after durationMillis, unless
   * {@link #stopCapturingVideo()} is not called meanwhile.
   *
   * @param file a file where the video will be saved
   * @param durationMillis video max duration
   *
   * @throws IllegalArgumentException if durationMillis is less than 500 milliseconds
   */
  public void startCapturingVideo(String path, String filename, long durationMillis) {
  	File file = new File(path,filename);
  	getObject().startCapturingVideo(file, durationMillis);
  }


  // TODO: pauseCapturingVideo and resumeCapturingVideo. There is mediarecorder.pause(), but API 24...


  /**
   * Stops capturing video, if there was a video record going on.
   * This will fire {@link CameraListener#onVideoTaken(File)}.
   */
  public void stopCapturingVideo() {
  	getObject().stopCapturingVideo();
  }


  /**
   * Returns the size used for the preview,
   * or null if it hasn't been computed (for example if the surface is not ready).
   * @return a Size
   */
  @SuppressWarnings("deprecation")
	public com.otaliastudios.cameraview.Size getPreviewSize() {
  	return getObject().getPreviewSize();
  }


  /**
   * Returns the size used for the capture,
   * or null if it hasn't been computed yet (for example if the surface is not ready).
   * @return a Size
   */
  @SuppressWarnings("deprecation")
	public com.otaliastudios.cameraview.Size getCaptureSize() {
  	return getObject().getCaptureSize();
  }


  /**
   * Returns the size used for capturing snapshots.
   * This is equal to {@link #getPreviewSize()}.
   *
   * @return a Size
   */
  public com.otaliastudios.cameraview.Size getSnapshotSize() {
      return getPreviewSize();
  }

	
	
}
